package dk.skat.sommerhus.transformation.javacallout;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Vector;
import java.nio.channels.FileChannel;

/**
 * <p>Implements Sommerhus message structure. Input is read from a filestream and written to a filestream</p>
 * <p><pre>
 * Individnavn:
 * Filnr.:
 * Individnr:
 * Individstrrelse 32 (min/max tegn)
 * Dataforkortelse	Lngde i bytes	Reprsentation	Bemrkninger
 * CVR-nr.	         8	            N	              For udlejningsbureauet
 * KALENDERR	       4	            N	              Kalenderret indberetningen vedrrer
 * KOMMUNEKODE	     3	            N	              Kode for den kommune, hvor fritidsboligen ligger. Hvis fritidsboligen ligger i udlandet angives 999 som kommunekode
 * LBENUMMER	       6	            N	              For fritidsboligen
 * LEJEBELB	      11	            N	              Bruttolejeindtgt i hele kroner
 *
 * </pre></p>
 * @author Ren Hjortskov Nielsen, SKAT
 * @version 0.1
 */
public class SommerhusXMLMessage extends SommerhusXMLMessageBase implements SommerhusXMLMessageInterface
{
  private long len = -1;

	public SommerhusXMLMessage(boolean debug, long len) {
		this.debug = debug;
		this.len = len;
	}

	/**
	 * Read data
   *
	 */
  public void readFromFile(FileInputStream fis, FileOutputStream fos)
		throws Exception
	{
		//if (debug) System.out.println("+readFromFile");
		// Reset message
		if (message!=null) { message.clear(); } else { message = new Vector(); }

    // Ignore first line, which is a text header
    readUntilFieldBytes(fis,0,0,82);
    readUntilFieldBytes(fis,0,0,82);
    readUntilFieldBytes(fis,0,0,82);
    readUntilFieldBytes(fis,0,0,82);
    readUntilFieldBytes(fis,0,0,82);
    readLinefeed(fis);

    message.addElement(getBytesFromString("<base64 xmlns:fase2.1=\"http://skat.dk/begrebsmodel/2009/01/15/\" xmlns:ctxt=\"http://skat.dk/begrebsmodel/xml/schemas/kontekst/2007/05/31/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"));
		// Create message...
    FileChannel fc = fis.getChannel();
    while (fc.position()<len) {
      message.addElement(getBytesFromString("<fase2.1:Angivelse><fase2.1:Angiver><fase2.1:VirksomhedCVRNummer>"));
  		message.addElement(readBytes(fis,0,8,1,null));  // CVRNR
  		message.addElement(getBytesFromString("</fase2.1:VirksomhedCVRNummer></fase2.1:Angiver><fase2.1:EjendomUdlejningIndkomstr>"));
  		message.addElement(readBytes(fis,0,4,1,null));  // R
  		message.addElement(getBytesFromString("</fase2.1:EjendomUdlejningIndkomstr><fase2.1:EjendomOplysninger><fase2.1:KommuneNummer>"));
  		message.addElement(readUntilFieldBytes(fis,0,0,3));  // KOMMUNEKODE
  		message.addElement(getBytesFromString("</fase2.1:KommuneNummer><fase2.1:EjendomNummer>"));
  		message.addElement(readUntilFieldBytes(fis,0,0,6));  // LBENUMMER
  		message.addElement(getBytesFromString("</fase2.1:EjendomNummer></fase2.1:EjendomOplysninger><fase2.1:EjendomUdlejningBruttoLejeIndtgtBelb>"));
  		message.addElement(readUntilFieldBytes(fis,0,0,11)); // LEJEBELB
  		message.addElement(getBytesFromString("</fase2.1:EjendomUdlejningBruttoLejeIndtgtBelb></fase2.1:Angivelse>"));
      // Discard LF
      readLinefeed(fis);
      writeToFile(fos);
    }
    message.addElement(getBytesFromString("</base64>"));
    writeToFile(fos);
		//if (debug) System.out.println("-readFromFile");
  }
}